// ../common/src/ast.ts
import { babelParse as _babelParse, walkIdentifiers } from "@vue/compiler-sfc";
function isCallOf(node, test) {
  return !!(node && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : test(node.callee.name)));
}
function checkInvalidScopeReference(node, method, setupBindings) {
  if (!node)
    return;
  walkIdentifiers(node, (id) => {
    if (setupBindings.includes(id.name))
      throw new SyntaxError(
        `\`${method}()\` in <script setup> cannot reference locally declared variables (${id.name}) because it will be hoisted outside of the setup() function.`
      );
  });
}

// ../common/src/constants.ts
var DEFINE_OPTIONS = "defineOptions";

// ../common/src/context.ts
import { MagicString } from "vue/compiler-sfc";

// ../common/src/vue.ts
import { compileScript, parse } from "@vue/compiler-sfc";
var parseSFC = (code, id) => {
  var _a;
  const { descriptor } = parse(code, {
    filename: id
  });
  const lang = (_a = descriptor.script || descriptor.scriptSetup) == null ? void 0 : _a.lang;
  let scriptCompiled;
  return {
    ...descriptor,
    lang,
    get scriptCompiled() {
      if (scriptCompiled)
        return scriptCompiled;
      return scriptCompiled = compileScript(descriptor, {
        id
      });
    }
  };
};
var addToScript = (ctx) => {
  const { scriptCode } = ctx;
  if (scriptCode.prepend.length + scriptCode.append.length === 0) {
    return;
  }
  const { sfc, s } = ctx;
  const { script, scriptSetup, lang } = sfc;
  if (script) {
    if (scriptCode.prepend) {
      s.appendRight(script.loc.start.offset, scriptCode.prepend);
      script.content = scriptCode.prepend + script.content;
    }
    if (scriptCode.append) {
      s.appendRight(script.loc.end.offset, scriptCode.append);
      script.content = script.content + scriptCode.append;
    }
  } else {
    const attrs = lang ? ` lang="${lang}"` : "";
    const content = `${scriptCode.prepend}
${scriptCode.append}`;
    s.prependLeft(0, `<script${attrs}>${content}<\/script>
`);
    sfc.script = {
      type: "script",
      content,
      attrs: (scriptSetup == null ? void 0 : scriptSetup.attrs) || {},
      loc: void 0
    };
  }
};

// ../common/src/context.ts
var initContext = (code, id) => {
  let s;
  let sfc;
  const ctx = {
    code,
    id,
    scriptCode: {
      prepend: "",
      append: ""
    },
    get s() {
      return s || (s = new MagicString(code));
    },
    set s(_s) {
      s = _s;
    },
    get sfc() {
      return sfc || (sfc = parseSFC(code, id));
    },
    set sfc(_sfc) {
      sfc = _sfc;
    }
  };
  return {
    ctx,
    getMagicString() {
      return s;
    },
    getSFC() {
      return sfc;
    }
  };
};
var finalizeContext = (ctx) => {
  addToScript(ctx);
};

// ../common/src/unplugin.ts
var getTransformResult = (s, id) => {
  if (s && s.original !== s.toString()) {
    return {
      code: s.toString(),
      get map() {
        return s.generateMap({
          source: id,
          includeContent: true
        });
      }
    };
  }
};

// src/core/utils.ts
var filterMarco = (stmts) => {
  return stmts.map((raw) => {
    let node = raw;
    if (raw.type === "ExpressionStatement")
      node = raw.expression;
    return isCallOf(node, DEFINE_OPTIONS) ? node : void 0;
  }).filter((node) => !!node);
};
var hasPropsOrEmits = (node) => node.properties.some(
  (prop) => (prop.type === "ObjectProperty" || prop.type === "ObjectMethod") && prop.key.type === "Identifier" && (prop.key.name === "props" || prop.key.name === "emits")
);

export {
  checkInvalidScopeReference,
  DEFINE_OPTIONS,
  parseSFC,
  initContext,
  finalizeContext,
  getTransformResult,
  filterMarco,
  hasPropsOrEmits
};
